@@ -2,9 +2,13 @@
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
+from . import environment
 from .config import ConfigurationError
 from .config import DOCKER_CONFIG_KEYS
 from .config import find
 from .config import load
 from .config import merge_environment
+from .config import merge_labels
 from .config import parse_environment
+from .config import parse_labels
+from .config import resolve_build_args
